/*
 * Copyright (C) 2013 Qbus B.V.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package nl.qbusict.cupboard;

import java.util.Map;

import nl.qbusict.cupboard.convert.Converter;
import nl.qbusict.cupboard.convert.ConverterHolder;

class BaseCompartment {
    protected final Map<Class<?>, ConverterHolder<?>> mConverters;

    protected BaseCompartment(Map<Class<?>, ConverterHolder<?>> converters) {
        this.mConverters = converters;
    }

    protected void createAllConverters() {
        for (ConverterHolder<?> holder : mConverters.values()) {
            holder.get();
        }
    }

    protected <T> Converter<T> getConverter(Class<T> clz) {
        @SuppressWarnings("unchecked")
        ConverterHolder<T> holder = (ConverterHolder<T>) mConverters.get(clz);
        if (holder == null) {
            throw new IllegalArgumentException("Class "+clz.toString()+" isn't registered.");
        }
        return holder.get();
    }
}
